/* ȥ륰եå 3D */

#ifdef MACOSX
#include <GLUT/glut.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#else
#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#endif

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "kame3d.h"
#include <GL/freeglut_ext.h>

#define PUBLIC
#define PRIVATE static

PRIVATE float step = TG_STEP;
PRIVATE float depth = 10.0;
//PRIVATE float depth = -10.0;
PRIVATE float rotx = 0.0;
PRIVATE float roty = 0.0;
PRIVATE float pan  = 0.0;
PRIVATE float tilt = 0.0;
PRIVATE float lightpos[] = {0.5, -0.5, 1.0, 0.0};
PRIVATE float lightamb[] = {0.5, 0.5, 0.5, 1.0};
PRIVATE float lightdif[] = {0.5, 0.5, 0.5, 1.0};
PRIVATE float lightspc[] = {0.2, 0.2, 0.2, 1.0};

PRIVATE void Fatal(const char *func, const char *msg)
{
	fprintf(stderr, "%s : %s\n", func, msg);
	exit(1);
}

PRIVATE void Debug(const char *msg)
{
	fprintf(stderr, "%s\n", msg);
}

PRIVATE void Help()
{
	Fatal("kame3d library", "produced by yanagawa@kushiro-ct.ac.jp");
}

PRIVATE int tg_timer;		// ޡưե饰
//PRIVATE int tg_clock;		// ޡ
double tg_clock;		// ޡ
PRIVATE int tg_interval;	// ޡֳ
PRIVATE int tg_mode;		// ­ˡ
PRIVATE int tg_show;		// ɽե饰
PRIVATE int tg_view;		// ˡ

// OpenGL 顼Ϣ

#define TG_ID_MAX	100
typedef struct {
	int ambient;		// 顼ID
	int diffuse;		// 顼ID
	int specular;		// 顼ID
	float shininess;
} tgMater;

PRIVATE float tg_color[TG_ID_MAX][4];
PRIVATE tgMater tg_mater[TG_ID_MAX];

PRIVATE void setVector(float v[], float x, float y, float z, float w)
{
	v[0] = x; v[1] = y; v[2] = z; v[3] = w;
}

PRIVATE int chkId(int id)
{
	if (id < 0) return(1);
	if (id > TG_ID_MAX) return(1);
	return (0);
}

PRIVATE int setColor(int id, float r, float g, float b, float a)
{
	if (chkId(id)) return(1);
	setVector(tg_color[id], r, g, b, a);
	return (0);
}

PRIVATE void useColor(int id)
{
	glColor4fv(tg_color[id]);
}

PRIVATE int setMater(int id, int amb, int dif, int spec, float shin)
{
	if (chkId(id)) return(1);
	tg_mater[id].ambient = amb;
	tg_mater[id].diffuse = dif;
	tg_mater[id].specular = spec;
	tg_mater[id].shininess = shin;
	return (0);
}

#define SURFACE	GL_FRONT_AND_BACK
PRIVATE void useMater(int id)
{
	glMaterialfv(SURFACE, GL_AMBIENT, tg_color[tg_mater[id].ambient]);
	glMaterialfv(SURFACE, GL_DIFFUSE, tg_color[tg_mater[id].diffuse]);
	glMaterialfv(SURFACE, GL_SPECULAR, tg_color[tg_mater[id].specular]);
	glMaterialf(SURFACE, GL_SHININESS, tg_mater[id].shininess);
}

// ᥪ֥ȴϢ

PRIVATE void setPos(tgState *obj, double x, double y)
{
	obj->x = x;
	obj->y = y;
}

PRIVATE void setSpeed(tgState *obj, double v)
{
	obj->v = v;
}

PRIVATE void updateDir(tgState *obj)
{
	obj->rad = obj->deg*M_PI/180.0;
	obj->c = cos(obj->rad);
	obj->s = sin(obj->rad);
}

PRIVATE void setDir(tgState *obj, double deg)
{
	obj->deg = deg;
	updateDir(obj);
}

PRIVATE void addDir(tgState *obj, double deg)
{
	obj->deg += deg;
	updateDir(obj);
}

PRIVATE void setPen(tgState *obj, int pen)
{
	if (pen < 0) pen = 0;
	else if (pen > 0) pen = 1;
	obj->pen = pen;
}

PRIVATE tgState *curState(tgObj *obj)
{
	return (obj->buf + obj->cur);
}

PRIVATE int nextState(tgObj *obj)
{
	tgState *p, *n;

	if (obj->cur >= TG_TRACE_MAX-1) return (1);

	p = curState(obj);
	obj->cur ++;
	n = curState(obj);
	*n = *p;

	return (0);
}

PUBLIC void tgInitObj(tgObj *obj, double x, double y, double deg)
{
	obj->cur = 0;
	setPos(obj->buf, x, y);
	setDir(obj->buf, deg);
	setSpeed(obj->buf, 1.0);
	setPen(obj->buf, 1);
	tgSetColor(obj, 1);
	tgSetPower(obj, 1.0);
	tgSetSpin(obj, 0);
}

PUBLIC tgObj *tgNewTurtle(double x, double y, double deg)
{
	tgObj *obj;

	obj = (tgObj *)malloc(sizeof(tgObj));
	if (obj == NULL) return (NULL);

	tgInitObj(obj, x, y, deg);

	return (obj);
}

PUBLIC void tgFreeTurtle(tgObj *obj)
{
	free(obj);
}

// ؿ

PUBLIC void tgPenUp(tgObj *obj)
{
	curState(obj)->pen = 0;
}

PUBLIC void tgPenDown(tgObj *obj)
{
	curState(obj)->pen = 1;
}

PUBLIC int tgSetColor(tgObj *obj, int color)
{
	if (chkId(color)) return (1);
	curState(obj)->color = color;
	return (0);
}

PUBLIC int tgSetPower(tgObj *obj, double power)
{
	curState(obj)->power = power;
	return (0);
}

PUBLIC int tgSetSpin(tgObj *obj, int spin)
{
	curState(obj)->spin = spin;
	return (0);
}

PUBLIC void tgGoto(tgObj *obj, double x, double y)
{
	tgState *p;
	int  pen;

	pen = curState(obj)->pen;
	tgPenUp(obj);

	nextState(obj);
	p = curState(obj);
	setPos(p, x, y);
	setPen(p, pen);
}

PUBLIC void tgHead(tgObj *obj, double deg)
{
	nextState(obj);
	setDir(curState(obj), deg);
}

PUBLIC void tgMove(tgObj *obj, double step)
{
	tgState *p;

	nextState(obj);
	p = curState(obj);
	p->x += step*p->c;
	p->y += step*p->s;
	p->v = step;
}

PUBLIC void tgTurn(tgObj *obj, double deg)
{
	tgState *p;

	nextState(obj);
	p = curState(obj);
	p->v = 0.0;
	addDir(p, deg);
}

PUBLIC double tgPosX(tgObj *obj)
{
	return (curState(obj)->x);
}

PUBLIC double tgPosY(tgObj *obj)
{
	return (curState(obj)->y);
}

// ̳Ѥ [0.0, 360.0) ֤
PUBLIC double tgDir(tgObj *obj)
{
	double deg;

	deg = curState(obj)->deg;
	if (deg < 0.0) deg -= (int)(deg/360.0 - 1)*360.0;
	else if (deg >= 360.0) deg -= (int)(deg/360.0)*360.0;

	return (deg);
}

PUBLIC int tgPen(tgObj *obj)
{
	return (curState(obj)->pen);
}

PUBLIC int tgColor(tgObj *obj)
{
	return (curState(obj)->color);
}

// Х

typedef struct {
	int    w, h;            // 
	double hw, hh;          // 2
} tgCanvas;

PRIVATE tgCanvas tg_canvas={10.0, 10.0, 5.0, 5.0};

PUBLIC void tgInitCanvas(int w, int h)
{
	tg_canvas.w = w;
	tg_canvas.h = h;
	tg_canvas.hw = w/2.0;
	tg_canvas.hh = h/2.0;
}

// OpenGL  

PRIVATE void hex1()
{
	double s60 = sqrt(3.0)/2.0;

	glPushMatrix();
		glBegin(GL_POLYGON);
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3f( 1.0,  0.0, 0.0);
			glVertex3f( 0.5,  s60, 0.0);
			glVertex3f(-0.5,  s60, 0.0);
			glVertex3f(-1.0,  0.0, 0.0);
			glVertex3f(-0.5, -s60, 0.0);
			glVertex3f( 0.5, -s60, 0.0);
		glEnd();
	glPopMatrix();
}

PRIVATE void hex2(double rot)
{
	double s60 = sqrt(3.0)/2.0;

	glPushMatrix();
		glRotatef(rot, 0.0, 0.0, 1.0);
		glTranslatef(0.0, -s60, 0.0);
		glRotatef(60.0, 1.0, 0.0, 0.0);
		glTranslatef(0.0, -s60, 0.0);
		glBegin(GL_POLYGON);
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3f( 0.5,  s60, 0.0);
			glVertex3f(-0.5,  s60, 0.0);
			glVertex3f(-0.75, 0.0, 0.0);
			glVertex3f( 0.75, 0.0, 0.0);
		glEnd();
	glPopMatrix();
}

PRIVATE void halfshell()
{
	glPushMatrix();
		glTranslatef(0.0, 0.0, 3.0/4.0);
		hex1();
		hex2(  0.0);
		hex2( 60.0);
		hex2(120.0);
		hex2(180.0);
		hex2(240.0);
		hex2(300.0);
	glPopMatrix();
}

PRIVATE void shell()
{
	useMater(2);
	glPushMatrix();
		glTranslatef(0.0, 0.0, 3.0/4.0);
		halfshell();
		glRotatef(180.0, 1.0, 0.0, 0.0);
		halfshell();
	glPopMatrix();
}

PRIVATE void head()
{
	useMater(3);
	glPushMatrix();
		glTranslatef(0.75, 0.0, 3.0/4.0+0.0);
		glRotatef(-45.0, 0.0, 1.0, 0.0);
		glTranslatef(0.75, 0.0, 0.0);
		glScalef(1.0, 0.5, 0.75);
		glutSolidCube(1.0);
	glPopMatrix();
}

PRIVATE void foot(double rot)
{
	useMater(3);
	glPushMatrix();
		glRotatef(rot, 0.0, 0.0, 1.0);
		glTranslatef(1.0, 0.0, 0.125);
		glScalef(1.0, 0.25, 0.25);
		glutSolidCube(1.0);
	glPopMatrix();
}

PRIVATE void turtle()
{
	glPushMatrix();
		glScalef(0.1, 0.1, 0.1);
		head();
		foot( 60.0);
		foot(120.0);
		foot(240.0);
		foot(300.0);
		shell();
	glPopMatrix();
}

PRIVATE tgObj **tg;
PRIVATE int   tg_num=0;

PRIVATE void updateTurtle(tgObj *obj)
{
	tgState	*p=obj->buf;
	double	f, dx, dy;
	if (tg_clock < obj->cur) {
		p += (int)tg_clock;
		f = tg_clock-(int)tg_clock;
		obj->x = (1.0-f)*(p->x) + f*((p+1)->x);
		obj->y = (1.0-f)*(p->y) + f*((p+1)->y);
		obj->deg = (1.0-f)*(p->deg) + f*((p+1)->deg);
		obj->rad = (1.0-f)*(p->rad) + f*((p+1)->rad);
		if (p->pen) {
			obj->z = 0.0;
		} else {
			dx = p->x - (p+1)->x;
			dy = p->y - (p+1)->y;
//			obj->z = sqrt(dx*dx + dy*dy)*(0.25 - (f-0.5)*(f-0.5));
		}
	} else {
		p += obj->cur;
		obj->x = p->x;
		obj->y = p->y;
		obj->z = 0.0;
		obj->deg = p->deg;
		obj->rad = p->rad;
	}
}

PRIVATE void drawTurtle(tgObj *obj)
{
	tgState *p=obj->buf;
	double  x, y, z, deg, f, dx, dy;
	double  d, spin;

	if (tg_clock < obj->cur) {
//		p += tg_clock;
		p += (int)tg_clock;
		f = tg_clock-(int)tg_clock;
		x = (1.0-f)*(p->x) + f*((p+1)->x);
		y = (1.0-f)*(p->y) + f*((p+1)->y);
		deg = (1.0-f)*(p->deg) + f*((p+1)->deg);

		if (p->pen) {
			z = 0.0;
		} else {
			dx = p->x - (p+1)->x;
			dy = p->y - (p+1)->y;
			d = sqrt(dx*dx + dy*dy);
			z = p->power*d*(0.25 - (f-0.5)*(f-0.5));
			if (d > 0.0) {
				spin = 360.0*f*p->spin;
			} else {
				spin = 0.0;
			}
		}
	} else {
		p += obj->cur;

		x = p->x;
		y = p->y;
		z = 0.0;
		deg = p->deg;
		spin = 0.0;
	}

	glPushMatrix();
		glTranslated(x, y, z);
		glRotated(deg + spin, 0.0, 0.0, 1.0);

		turtle();
	glPopMatrix();
}

PRIVATE void drawTrace(tgObj *obj)
{
	int   i, n;
	int   prev = 0;
	tgState *p = obj->buf;
	double f, x, y, rad;

	if (tg_clock < obj->cur) {
		n = tg_clock;
		f = tg_clock-(int)tg_clock;
	} else {
		n = obj->cur;
		f= 0.0;
	}

	if (tg_mode) {
//		useMater(p->color);
		for (i = 0; i <= n; i++) {
			if (p->pen > prev) glBegin(GL_QUAD_STRIP);

//			if ((p->pen|prev) && (p->v != 0.0)) {
			if (p->pen|prev) {
				if (p->v > 0.0) {
					glNormal3d(-(p->s), (p->c), 0.0);
				} else {
					glNormal3d((p->s), -(p->c), 0.0);
				}
				glVertex3d((p->x), (p->y), 0.0);
				glVertex3d((p->x), (p->y), 0.1);
				useMater(p->color);
			}

			if (p->pen < prev) glEnd();

			prev = p->pen;
			p++;
		}
		if (f > 0.0) {
			p--;
			if ((p->pen|prev) && ((p+1)->v != 0.0)) {
				x = (1.0-f)*(p->x) + f*((p+1)->x);
				y = (1.0-f)*(p->y) + f*((p+1)->y);
				rad = (1.0-f)*(p->rad) + f*((p+1)->rad);
				if ((p+1)->v > 0.0) {
					glNormal3d(-sin(rad), cos(rad), 0.0);
				} else {
					glNormal3d(sin(rad), -cos(rad), 0.0);
				}
				glVertex3d(x, y, 0.0);
				glVertex3d(x, y, 0.1);
				useMater(p->color);
			}
		}
		if (prev) glEnd();
	} else {
		glDisable(GL_LIGHTING);
		for (i = 0; i <= n; i++) {
			if (p->pen > prev) glBegin(GL_LINE_STRIP);

//			if ((p->pen|prev) && (p->v != 0.0)) {
			if ((p->pen|prev)) {
				glNormal3d(-(p->s), (p->c), 0.0);
				glVertex3d((p->x), (p->y), 0.01);
				useColor(p->color);
			}

			if (p->pen < prev) glEnd();

			prev = p->pen;
			p++;
		}
		if (f > 0.0) {
			p--;
			if ((p->pen|prev) && ((p+1)->v != 0.0)) {
				x = (1.0-f)*(p->x) + f*((p+1)->x);
				y = (1.0-f)*(p->y) + f*((p+1)->y);
				rad = (1.0-f)*(p->rad) + f*((p+1)->rad);
				glNormal3d(-sin(rad), cos(rad), 0.0);
				glVertex3d(x, y, 0.01);
				useColor(p->color);
			}
		}
		if (prev) glEnd();
		glEnable(GL_LIGHTING);
	}
}

PRIVATE void drawCanvas()
{
	int   i, j;
	float x, y0, y1;

	glPushMatrix();
		useMater(11);
		for (i = 0; i < tg_canvas.h; i++) {
			glBegin(GL_QUAD_STRIP);
				glNormal3f(0.0, 0.0, 1.0);
				y0 = -tg_canvas.hh + i;
				y1 = -tg_canvas.hh + (i+1);
				for (j = 0; j <= tg_canvas.w; j++) {
					x = -tg_canvas.hw + j;
					glVertex3f(x, y1, 0.0);
					glVertex3f(x, y0, 0.0);
					useMater(10+(i+j)%2);
				}
			glEnd();
		}
	glPopMatrix();
}

PRIVATE void light()
{
		glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
		glLightfv(GL_LIGHT0, GL_AMBIENT, lightamb);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, lightdif);
		glLightfv(GL_LIGHT0, GL_SPECULAR, lightspc);
}

PRIVATE void camera()
{
	static double x, y, z;
	static double rad;
	static double dx, dy;

	x = tg_kame->x;
	y = tg_kame->y;
	z = tg_kame->z;

	switch (tg_view) {
	case 0 :	// Ļ
		gluLookAt(0.0, -depth, depth, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);			break;
	case 1 :	// 
		gluLookAt(0.0, -depth, depth, x, y, z, 0.0, 0.0, 1.0);
		break;
	case 2 :	// ;Ρʥܡɡ
		rad = tg_kame->rad;
		dx = cos(rad);
		dy = sin(rad);
		gluLookAt(x-2.0*dx, y-2.0*dy, z+0.5,
			x+4.0*dx, y+4.0*dy, 0.0,
			0.0, 0.0, 1.0);
		break;
	}
}

PRIVATE void display()
{
	int i;

	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glLineWidth(3);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glPushMatrix();
		light();
		camera();

		if (tg_view == 0) {
//			glTranslatef(0.0, -0.5, depth);
			glTranslatef(0.0, tilt, 0.0);
			glTranslatef(pan, 0.0, 0.0);
//			glRotatef(roty-45.0, 1.0, 0.0, 0.0);
			glRotatef(roty, 1.0, 0.0, 0.0);
			glRotatef(rotx, 0.0, 0.0, 1.0);
		}

		for (i = 0; i < tg_num; i++) {
			if (tg_show) drawTurtle(tg[i]);
			drawTrace(tg[i]);
		}
		drawCanvas();
	glPopMatrix(); 

	glutSwapBuffers();
}

PRIVATE void toggleView()
{
	tg_view = (tg_view+1)%3;
}

PRIVATE void update()
{
	int  i;

	for (i = 0; i < tg_num; i++) {
		updateTurtle(tg[i]);
	}
}

PRIVATE void timer(int val)
{
	if (tg_timer == 0) return;

//	tg_clock++;
//tg_clock += TG_STEP;
tg_clock += step;
	if (tg_clock >= TG_TRACE_MAX) return;

	update();

	glutTimerFunc(tg_interval, timer, 0);
	glutPostRedisplay();
}

PRIVATE void resetTimer()
{
	tg_clock = 0;
	tg_timer = 0;
}

PRIVATE void startTimer()
{
	if (tg_timer) return;
	if (tg_interval == 0) return;

	glutTimerFunc(tg_interval, timer, 0);
	tg_timer = 1;
}

PRIVATE void stopTimer()
{
	tg_timer = 0;
}

PRIVATE void endTimer()
{
	tg_timer = 0;
	tg_clock = TG_TRACE_MAX;
}

PRIVATE void toggleTimer()
{
	if (tg_timer) {
		stopTimer();
	} else {
		startTimer();
	}
}

PRIVATE void toggleTurtle()
{
	tg_show = (tg_show+1)%2;
}

PRIVATE void toggleMode()
{
	tg_mode = (tg_mode+1)%2;
}

#define KEY_ESC 27 

PRIVATE void keyboard(unsigned char key, int x, int y)
{
	switch (key) {
	case KEY_ESC :
//	case 'q' : exit(0);
	case 'q' : glutLeaveMainLoop(); break;
	case ' ' : depth -= 0.5; break;
	case 'b' : depth += 0.5; break;
	case 'l' : pan -= 0.25; break;
	case 'h' : pan += 0.25; break;
	case 'j' : tilt += 0.25; break;
	case 'k' : tilt -= 0.25; break;

	case 'r' : resetTimer(); break;
	case 's' : toggleTimer(); break;
	case 'e' : endTimer(); break;
	case 't' : toggleTurtle(); break;
	case 'c' : toggleMode(); break;
	case 'v' : toggleView(); break;

	case '-' : step /= 2.0; break;
	case '+' : step *= 2.0; break;
	}
	glutPostRedisplay();
}

PRIVATE void keypad(int key, int x, int y)
{
	switch (key) {
	case GLUT_KEY_UP :    roty -= 2.0; break;
	case GLUT_KEY_DOWN :  roty += 2.0; break;
	case GLUT_KEY_LEFT :  rotx -= 2.0; break;
	case GLUT_KEY_RIGHT : rotx += 2.0; break;
	}
	glutPostRedisplay();
}

PRIVATE void reshape(int w, int h)
{
	double aspect = (double)w/(double)h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(30.0, aspect, 1.0, 100.0);
	glViewport(0, 0, w, h);

	glutPostRedisplay();
}

PRIVATE void idle()
{
	glutPostRedisplay();
}

PRIVATE void initGL(int argc, char **argv, char *title)
{
	int width=800, height=450;

//	glutInitWindowPosition(0, 0); 
	glutInitWindowSize(width, height);
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE|GLUT_DEPTH);
	glutCreateWindow(title);

	glClearColor(0.5, 0.5, 1.0, 1.0);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(keypad);
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
//	glutIdleFunc(idle);
	glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE, GLUT_ACTION_CONTINUE_EXECUTION);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glShadeModel(GL_FLAT);
//	glShadeModel(GL_SMOOTH);

	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 1);

	glEnable(GL_NORMALIZE);
}

PUBLIC void tgInit(int argc, char **argv, char *title)
{
	int i;

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-help") == 0) Help();
	}
	if ((title == NULL) || (strlen(title) == 0)) title = "Kame3D";
	initGL(argc, argv, title);

	setColor(0, 0.5, 0.5, 0.5, 1.0);
	setColor(1, 1.0, 0.5, 0.5, 1.0);
	setColor(2, 0.5, 1.0, 0.5, 1.0);
	setColor(3, 1.0, 1.0, 0.5, 1.0);
	setColor(4, 0.5, 0.5, 1.0, 1.0);
	setColor(5, 1.0, 0.5, 1.0, 1.0);
	setColor(6, 0.5, 1.0, 1.0, 1.0);
	setColor(7, 0.7, 0.7, 0.7, 1.0);

	setMater(0, 0, 0, 0, 15.0);
	setMater(1, 1, 1, 1, 15.0);
	setMater(2, 2, 2, 2, 15.0);
	setMater(3, 3, 3, 3, 15.0);
	setMater(4, 4, 4, 4, 15.0);
	setMater(5, 5, 5, 5, 15.0);
	setMater(6, 6, 6, 6, 15.0);
	setMater(7, 7, 7, 7, 15.0);

	setMater(10, 0, 0, 0, 5.0);
	setMater(11, 7, 7, 0, 5.0);

	tg_mode = 0;
	tg_show = 1;
	tg_view = 0;

//	tgInitCanvas(10.0, 10.0);
	resetTimer();

	srand(time(NULL));
}

PUBLIC void tgPlay(int msec, tgObj *obj[], int n)
{
	tg_interval = msec;
	if (tg_interval == 0) {
		stopTimer();
	} else {
		startTimer();
	}
	tg = obj;
	tg_num = n;
	tg_kame = obj[0];
	glutMainLoop();
//	glutMainLoopEvent();
}

PUBLIC int tgStep(tgObj *obj)
{
	return (obj->cur);
}

// ʰ״ؿ

tgObj *tg_kame=NULL;

void InitTurtle(double x, double y, double deg)
{
	if (tg_kame != NULL) tgFreeTurtle(tg_kame);
	tg_kame = tgNewTurtle(x, y, deg);
}

void Init(char *title)
{
	tgInit(0, NULL, title);
	tgInitCanvas(10.0, 10.0);
	InitTurtle(0.0, 0.0, 0.0);
}

// 0 ʾ塤n ̤
PUBLIC int Rand(int n)
{
	return ((int)((double)n*(double)rand()/(RAND_MAX + 1.0)));
}

